/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.sanskritlibrary.textservice;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;

/**
 *
 * @author ralph
 */
public class ServiceMain extends SanskritLibraryServlet {


    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        
        String texttype = request.getParameter("texttype");
        if ("forTranslation".equals(texttype)) {
            doGetForTranslation(request, response);
        } else if ("forManuscript".equals(texttype)) {
            doGetForManuscript(request, response);            
        } else {
            writeResult(response, "<h3>No texttype parameter or an illegal one</h3>", "text/html");
        }
    }
    

    private void doGetForTranslation(HttpServletRequest request, HttpServletResponse response) 
            throws ServletException, IOException {
       
        String sandhiFlag = null;
        String dataDirectory = getServletContext().getInitParameter("dataDirectory");
        if (!dataDirectory.endsWith("/")) {
            dataDirectory += "/";
        }
        
        String source = request.getParameter("source");
        String textsDirectory = dataDirectory + source + "/";
        String name = request.getParameter("name");
        
        if (name != null) {
            try {
                sandhiFlag = request.getParameter("sandhiFlag");

                String slpName = textsDirectory + name + ".slp";
                if (new File(slpName).exists() == false) {
                    slpName = textsDirectory + name + ".slp1";
                }
                BufferedReader br = IOAssistance.openReader(slpName);
                String line;
                StringBuilder sb = new StringBuilder();
                sb.append("{\n");
                sb.append("\"lines\":[\n");
                
                String delim = "";
                while ((line = br.readLine())!= null) {
                    sb.append(String.format("%s%s", 
                            delim, fixup(line, sandhiFlag)));  // Assume that sandhi has been applied
                    delim = ",\n";
                }
                sb.append("]\n}");
                br.close();
                this.writeResult(response, sb.toString(), "text/json");
            } catch (Exception ex) {
                writeResult(response, ex.getMessage());
            }
        } else {
            writeResult(response, "<h3>No name parameter</h3>", "text/html");
        }
        
    }
    
    private void doGetForManuscript(HttpServletRequest request, HttpServletResponse response) 
            throws ServletException, IOException {
        
        String sandhiFlag = null;
        String dataDirectory = getServletContext().getInitParameter("dataDirectory");
        if (!dataDirectory.endsWith("/")) {
            dataDirectory += "/";
        }
        
        String source = request.getParameter("source");
        String textsDirectory = dataDirectory + source + "/";
        String name = request.getParameter("name");
        
        String title = request.getParameter("title");
        if (title == null) {
            title = name;
        }
        
        if (name != null) {
            try {
                sandhiFlag = request.getParameter("sandhiFlag");
                if (sandhiFlag == null) {
                    sandhiFlag = "";            // lines already begin with sandhi flag character
                }

                String slpName = textsDirectory + name + ".slp";
                if (new File(slpName).exists() == false) {
                    slpName = textsDirectory + name + ".slp1";
                }
                BufferedReader br = IOAssistance.openReader(slpName);
                String line;
                StringBuilder sb = new StringBuilder();
                sb.append("\"lines\":[\n");
                String delim = "";
                while ((line = br.readLine())!= null) {
                    line = prepareForEnclosureInQuotes(line);
                    if (line.charAt(0) == '%') {            // do not use sandhiFlag prefix
                        sb.append(String.format("%s\"%s\"", 
                                delim, line));                     
                    } else {
                        sb.append(String.format("%s\"%s%s\"", 
                                delim, sandhiFlag, line)); 
                    }
                    delim = ",\n";
                }
                sb.append("]");
                br.close();
                this.writeResult(response, sb.toString(), "text/text");
            } catch (Exception ex) {
                writeResult(response, ex.getMessage());
            }
        } else {
            writeResult(response, "<h3>No name parameter</h3>", "text/html");
        }
        
    }
    
    private String prepareForEnclosureInQuotes(String line) {
        if (line.contains("\\")) {          // do this first before escaping anything else
            line = line.replaceAll("\\\\", "\\\\\\\\");
        }

        line = line.replaceAll("\"", "\\\\\"");

        return line;
    }

    private String fixup(String line, String sandhiFlag) {
        line = prepareForEnclosureInQuotes(line);
        String[] words = line.split("\\s+");
        StringBuilder sb = new StringBuilder();
        sb.append("[");
        String delim = "";
        int numwords = words.length;
        for (int i=0; i<numwords; i++) {
            String word = words[i];
            if (sandhiFlag != null && i == 0) {
                sb.append(String.format("\"%s%s\"", sandhiFlag, word));                
            } else {
                sb.append(String.format("%s\"%s\"", delim, word));
            }
            delim = ",";
        }
        sb.append("]");
        
        return sb.toString();
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
